'use strict';

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;
exports['default'] = write_assets;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _helpers = require('../helpers');

var _common = require('../common');

// writes webpack-assets.json file, which contains assets' file paths

function write_assets(json, options, log) {
	// take the passed in options
	options = _helpers.clone(options);

	// make webpack stats accessible for asset functions (parser, naming, filter)
	options.webpack_stats = json;

	log.debug('running write assets webpack plugin');

	var development = options.development;

	if (development) {
		log.debug(' (development mode is on)');
	}

	// create all the folders in the path if they don't exist
	_mkdirp2['default'].sync(_path2['default'].dirname(options.webpack_assets_path));

	// write webpack stats json for debugging purpose
	// (and for evaluating webpack module require()s)

	// path to webpack stats file
	var webpack_stats_file_path = _common.webpack_stats_file_path(options.webpack_assets_path);

	// write webpack stats file
	log.debug('writing webpack stats to ' + webpack_stats_file_path);
	// format the JSON for better readability in development mode
	var webpack_stats_json = development ? JSON.stringify(json, null, 2) : JSON.stringify(json);
	// write the file
	_fs2['default'].writeFileSync(webpack_stats_file_path, webpack_stats_json);

	// the output object with assets
	var output = options.output;

	// populate the output object with assets
	populate_assets(output, json, options, log);

	// write webpack assets info file
	log.debug('writing webpack assets info to ' + options.webpack_assets_path);
	// format the JSON for better readability if in debug mode
	var assets_info = development ? JSON.stringify(output, null, 2) : JSON.stringify(output);
	// write the file
	_fs2['default'].writeFileSync(options.webpack_assets_path, assets_info);
}

// populates the output object with assets
function populate_assets(output, json, options, log) {
	// for each chunk name ("main", "common", ...)
	_Object$keys(json.assetsByChunkName).forEach(function (name) {
		log.debug('getting javascript and styles for chunk "' + name + '"');

		// get javascript chunk real file path

		var javascript = get_assets(name, 'js')[0];
		// the second asset is usually a source map

		if (javascript) {
			log.debug(' (got javascript)');
			output.javascript[name] = javascript;
		}

		// get style chunk real file path

		var style = get_assets(name, 'css')[0];
		// the second asset is usually a source map

		if (style) {
			log.debug(' (got style)');
			output.styles[name] = style;
		}
	});

	// gets asset paths by name and extension of their chunk
	function get_assets(name) {
		var extension = arguments.length <= 1 || arguments[1] === undefined ? 'js' : arguments[1];

		var chunk = json.assetsByChunkName[name];

		// a chunk could be a string or an array, so make sure it is an array
		if (!Array.isArray(chunk)) {
			chunk = [chunk];
		}

		return chunk
		// filter by extension
		.filter(function (name) {
			return _path2['default'].extname(name) === '.' + extension;
		})
		// adjust the real path (can be http, filesystem)
		.map(function (name) {
			return options.assets_base_url + name;
		});
	}

	// // output assets for all application javascript entry points
	// Object.keys(this.options.entry).forEach(chunk_name =>
	// {
	// 	let entry = this.options.entry[chunk_name]
	// 	if (Array.isArray(entry))
	// 	{
	// 		entry = entry[entry.length - 1]
	// 	}
	//
	// 	output.javascript[entry] = get_assets(chunk_name, 'js')[0]
	// 	// the second asset is usually a source map
	// })

	// // omit node_modules contents and internal webpack modules
	// const modules = json.modules.filter(module =>
	// {
	// 	return module.name.indexOf('.') === 0 && module.name.indexOf('./~/') !== 0
	// 	// return module.name.indexOf('./~/') !== 0 && module.name.indexOf('(webpack)') !== 0
	// })

	// one can supply a custom filter
	var default_filter = function default_filter(module, regular_expression) {
		return regular_expression.test(module.name);
	};
	// one can supply a custom namer
	var default_asset_path = function default_asset_path(module) {
		return module.name;
	};
	// one can supply a custom parser
	var default_parser = function default_parser(module) {
		return module.source;
	};

	var _loop = function () {
		if (_isArray) {
			if (_i >= _iterator.length) return 'break';
			_ref = _iterator[_i++];
		} else {
			_i = _iterator.next();
			if (_i.done) return 'break';
			_ref = _i.value;
		}

		var asset_type = _ref;

		var asset_description = options.assets[asset_type];

		// one can supply his own filter
		var filter = asset_description.filter || default_filter; //.bind(this)
		// one can supply his own parser
		var parser = asset_description.parser || default_parser; //.bind(this)
		// one can supply his own namer
		var extract_asset_path = asset_description.path || default_asset_path; //.bind(this)

		// // parser is required
		// if (!asset_description.parser)
		// {
		// 	throw new Error(`"parser" function is required for assets type "${asset_type}". See the Configuration section of the README for explanation.`)
		// }

		log.debug('populating assets of type "' + asset_type + '"');

		// timer start
		var began_at = new Date().getTime();

		// get real paths for all the files from this asset type
		json.modules
		// take just modules of this asset type
		.filter(function (module) {
			// check that this asset is of the asset type
			if (!filter(module, options.regular_expressions[asset_type], options, log)) {
				return false;
			}

			// guard against an empty source.
			if (!module.source) {
				log.error('Module "' + module.name + '" has no source. Maybe Webpack compilation of this module failed. Skipping this asset.');
				return false;
			}

			// include this asset
			return true;
		}).reduce(function (set, module) {
			// determine asset real path
			var asset_path = extract_asset_path(module, options, log);

			// (no need to resolve require() paths because it just works as is)
			//
			// const resolve_require_paths = (module_source) =>
			// {
			// 	return module_source.replace(/(\s)require\("([^"]+)"\)/g, function(match, space_before, required_path)
			// 	{
			// 		const asset_folder = path.dirname(asset_path)
			// 		const required_file_path = path.join(asset_folder, required_path)
			//
			// 		const required_file_absolute_path = path.resolve(options.project_folder, required_file_path)
			// 		const node_modules_path = path.join(options.project_folder, 'node_modules')
			//
			// 		// if it's a require() call for a file inside "node_modules",
			// 		// then adjust the required path accordingly
			// 		if (required_file_absolute_path.indexOf(node_modules_path) === 0)
			// 		{
			// 			// account for the last '/'
			// 			required_path = required_path.slice(node_modules_path.length + 1)
			// 		}
			// 		else
			// 		{
			// 			required_path = './' + replace_all(required_file_path, path.sep, '/')
			// 		}
			//
			// 		return space_before + `require("${required_path}")`
			// 	})
			// }

			// asset module source, or asset content (or whatever else)
			var parsed_asset = parser(module, options, log);

			log.trace('Adding assset "' + asset_path + '", module id ' + module.id + ' (in webpack-stats.debug.json)');

			// check for naming collisions (just in case)
			if (_helpers.exists(set[asset_path])) {
				log.error('-----------------------------------------------------------------');
				log.error('Asset with path "' + asset_path + '" was overwritten because of path collision.');
				log.error('Use the "filter" function of this asset type to narrow the results.');
				log.error('Previous asset with this path:');
				log.error(set[asset_path]);
				log.error('New asset with this path:');
				log.error(parsed_asset);
				log.error('-----------------------------------------------------------------');
			}

			// add this asset to the list
			set[asset_path] = parsed_asset;
			// continue
			return set;
		}, output.assets);

		// timer stop
		log.debug(' time taken: ' + (new Date().getTime() - began_at) + ' ms');
	};

	// put assets of all types into a single object for speeding up lookup by asset path

	// for each user specified asset type
	for (var _iterator = _Object$keys(options.assets), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _getIterator(_iterator);;) {
		var _ref;

		var _ret = _loop();

		if (_ret === 'break') break;
	}
}
module.exports = exports['default'];
//# sourceMappingURL=write assets.js.map